@@ -5,18 +5,18 @@ if False:  # MYPY
 
 _package_data = dict(
     full_package_name='ruamel.yaml',
-    version_info=(0, 17, 28),
-    __version__='0.17.28',
-    version_timestamp='2023-05-26 13:15:05',
+    version_info=(0, 18, 5),
+    __version__='0.18.5',
+    version_timestamp='2023-11-03 08:54:26',
     author='Anthon van der Neut',
     author_email='a.van.der.neut@ruamel.eu',
     description='ruamel.yaml is a YAML parser/emitter that supports roundtrip preservation of comments, seq/map flow style, and map key order',  # NOQA
     entry_points=None,
     since=2014,
     extras_require={
-        ':platform_python_implementation=="CPython" and python_version<"3.12"': ['ruamel.yaml.clib>=0.2.7'],  # NOQA
+        ':platform_python_implementation=="CPython" and python_version<"3.13"': ['ruamel.yaml.clib>=0.2.7'],  # NOQA
         'jinja2': ['ruamel.yaml.jinja2>=0.2'],
-        'docs': ['ryd'],
+        'docs': ['ryd', 'mercurial>5.7'],
     },
     classifiers=[
         'Programming Language :: Python :: 3 :: Only',
@@ -25,6 +25,7 @@ _package_data = dict(
         'Programming Language :: Python :: 3.9',
         'Programming Language :: Python :: 3.10',
         'Programming Language :: Python :: 3.11',
+        'Programming Language :: Python :: 3.12',
         'Programming Language :: Python :: Implementation :: CPython',
         'Topic :: Software Development :: Libraries :: Python Modules',
         'Topic :: Text Processing :: Markup',
@@ -38,8 +39,7 @@ _package_data = dict(
         fl8excl='_test/lib,branch_default',
     ),
     # universal=True,
-    python_requires='>=3',
-    rtfd='yaml',
+    python_requires='>=3.7',
 )  # type: Dict[Any, Any]
 
 
