@@ -14,6 +14,7 @@ from ruamel.yaml.compat import ordereddict
 from ruamel.yaml.compat import MutableSliceableSequence, nprintf  # NOQA
 from ruamel.yaml.scalarstring import ScalarString
 from ruamel.yaml.anchor import Anchor
+from ruamel.yaml.tag import Tag
 
 from collections.abc import MutableSet, Sized, Set, Mapping
 
@@ -23,6 +24,7 @@ from typing import Any, Dict, Optional, List, Union, Optional, Iterator  # NOQA
 __all__ = ['CommentedSeq', 'CommentedKeySeq',
            'CommentedMap', 'CommentedOrderedMap',
            'CommentedSet', 'comment_attrib', 'merge_attrib',
+           'TaggedScalar',
            'C_POST', 'C_PRE', 'C_SPLIT_ON_FIRST_BLANK', 'C_BLANK_LINE_PRESERVE_SPACE',
            ]
 # fmt: on
@@ -79,7 +81,6 @@ comment_attrib = '_yaml_comment'
 format_attrib = '_yaml_format'
 line_col_attrib = '_yaml_line_col'
 merge_attrib = '_yaml_merge'
-tag_attrib = '_yaml_tag'
 
 
 class Comment:
@@ -220,6 +221,9 @@ class Format:
             return default
         return self._flow_style
 
+    def __repr__(self) -> str:
+        return f'Format({self._flow_style})'
+
 
 class LineCol:
     """
@@ -264,19 +268,6 @@ class LineCol:
         return f'LineCol({self.line}, {self.col})'
 
 
-class Tag:
-    """store tag information for roundtripping"""
-
-    __slots__ = ('value',)
-    attrib = tag_attrib
-
-    def __init__(self) -> None:
-        self.value = None
-
-    def __repr__(self) -> Any:
-        return f'{self.__class__.__name__}({self.value!r})'
-
-
 class CommentedBase:
     @property
     def ca(self):
@@ -380,7 +371,7 @@ class CommentedBase:
         return getattr(self, Format.attrib)
 
     def yaml_add_eol_comment(
-        self, comment: Any, key: Optional[Any] = NotNone, column: Optional[Any] = None
+        self, comment: Any, key: Optional[Any] = NotNone, column: Optional[Any] = None,
     ) -> None:
         """
         there is a problem as eol comments should start with ' #'
@@ -442,10 +433,14 @@ class CommentedBase:
             setattr(self, Tag.attrib, Tag())
         return getattr(self, Tag.attrib)
 
-    def yaml_set_tag(self, value: Any) -> None:
-        self.tag.value = value
+    def yaml_set_ctag(self, value: Tag) -> None:
+        setattr(self, Tag.attrib, value)
 
-    def copy_attributes(self, t: Any, memo: Any = None) -> None:
+    def copy_attributes(self, t: Any, memo: Any = None) -> Any:
+        """
+        copies the YAML related attributes, not e.g. .values
+        returns target
+        """
         # fmt: off
         for a in [Comment.attrib, Format.attrib, LineCol.attrib, Anchor.attrib,
                   Tag.attrib, merge_attrib]:
@@ -454,6 +449,7 @@ class CommentedBase:
                     setattr(t, a, copy.deepcopy(getattr(self, a, memo)))
                 else:
                     setattr(t, a, getattr(self, a))
+        return t
         # fmt: on
 
     def _yaml_add_eol_comment(self, comment: Any, key: Any) -> None:
@@ -576,7 +572,7 @@ class CommentedSeq(MutableSliceableSequence, list, CommentedBase):  # type: igno
             list.__init__(self, [x[0] for x in tmp_lst])
         else:
             tmp_lst = sorted(
-                zip(map(key, list.__iter__(self)), range(len(self))), reverse=reverse
+                zip(map(key, list.__iter__(self)), range(len(self))), reverse=reverse,
             )
             list.__init__(self, [list.__getitem__(self, x[1]) for x in tmp_lst])
         itm = self.ca.items
@@ -714,7 +710,7 @@ class CommentedMap(ordereddict, CommentedBase):
         ordereddict.__init__(self, *args, **kw)
 
     def _yaml_add_comment(
-        self, comment: Any, key: Optional[Any] = NotNone, value: Optional[Any] = NotNone
+        self, comment: Any, key: Optional[Any] = NotNone, value: Optional[Any] = NotNone,
     ) -> None:
         """values is set to key to indicate a value attachment of comment"""
         if key is not NotNone:
@@ -1098,7 +1094,7 @@ class CommentedSet(MutableSet, CommentedBase):  # type: ignore  # NOQA
             self |= values
 
     def _yaml_add_comment(
-        self, comment: Any, key: Optional[Any] = NotNone, value: Optional[Any] = NotNone
+        self, comment: Any, key: Optional[Any] = NotNone, value: Optional[Any] = NotNone,
     ) -> None:
         """values is set to key to indicate a value attachment of comment"""
         if key is not NotNone:
@@ -1141,7 +1137,9 @@ class TaggedScalar(CommentedBase):
         self.value = value
         self.style = style
         if tag is not None:
-            self.yaml_set_tag(tag)
+            if isinstance(tag, str):
+                tag = Tag(suffix=tag)
+            self.yaml_set_ctag(tag)
 
     def __str__(self) -> Any:
         return self.value
@@ -1152,6 +1150,9 @@ class TaggedScalar(CommentedBase):
     def __getitem__(self, pos: int) -> Any:
         return self.value[pos]
 
+    def __repr__(self) -> str:
+        return f'TaggedScalar(value={self.value!r}, style={self.style!r}, tag={self.tag!r})'
+
 
 def dump_comments(d: Any, name: str = "", sep: str = '.', out: Any = sys.stdout) -> None:
     """
@@ -1170,5 +1171,5 @@ def dump_comments(d: Any, name: str = "", sep: str = '.', out: Any = sys.stdout)
         out.write(f'{d.ca!r}\n')
         for idx, k in enumerate(d):
             dump_comments(
-                k, name=(name + sep + str(idx)) if name else str(idx), sep=sep, out=out
+                k, name=(name + sep + str(idx)) if name else str(idx), sep=sep, out=out,
             )
