@@ -81,6 +81,7 @@ from ruamel.yaml.scanner import Scanner, RoundTripScanner, ScannerError  # NOQA
 from ruamel.yaml.scanner import BlankLineComment
 from ruamel.yaml.comments import C_PRE, C_POST, C_SPLIT_ON_FIRST_BLANK
 from ruamel.yaml.compat import nprint, nprintf  # NOQA
+from ruamel.yaml.tag import Tag
 
 from typing import Any, Dict, Optional, List, Optional  # NOQA
 
@@ -182,6 +183,7 @@ class Parser:
     def parse_implicit_document_start(self) -> Any:
         # Parse an implicit document.
         if not self.scanner.check_token(DirectiveToken, DocumentStartToken, StreamEndToken):
+            # don't need copy, as an implicit tag doesn't add tag_handles
             self.tag_handles = self.DEFAULT_TAGS
             token = self.scanner.peek_token()
             start_mark = end_mark = token.start_mark
@@ -273,7 +275,7 @@ class Parser:
 
     def parse_document_content(self) -> Any:
         if self.scanner.check_token(
-            DirectiveToken, DocumentStartToken, DocumentEndToken, StreamEndToken
+            DirectiveToken, DocumentStartToken, DocumentEndToken, StreamEndToken,
         ):
             event = self.process_empty_scalar(self.scanner.peek_token().start_mark)
             self.state = self.states.pop()
@@ -289,7 +291,7 @@ class Parser:
             if token.name == 'YAML':
                 if yaml_version is not None:
                     raise ParserError(
-                        None, None, 'found duplicate YAML directive', token.start_mark
+                        None, None, 'found duplicate YAML directive', token.start_mark,
                     )
                 major, minor = token.value
                 if major != 1:
@@ -312,11 +314,14 @@ class Parser:
         else:
             value = yaml_version, None
         if self.loader is not None and hasattr(self.loader, 'tags'):
+            # ToDo: this is used to keep a single loaded file from losing its version
+            # info, but  it affects following versions that have no explicit directive
             self.loader.version = yaml_version
             if self.loader.tags is None:
                 self.loader.tags = {}
             for k in self.tag_handles:
                 self.loader.tags[k] = self.tag_handles[k]
+                self.loader.doc_infos[-1].tags.append((k, self.tag_handles[k]))
         for key in self.DEFAULT_TAGS:
             if key not in self.tag_handles:
                 self.tag_handles[key] = self.DEFAULT_TAGS[key]
@@ -347,8 +352,13 @@ class Parser:
     def parse_block_node_or_indentless_sequence(self) -> Any:
         return self.parse_node(block=True, indentless_sequence=True)
 
-    def transform_tag(self, handle: Any, suffix: Any) -> Any:
-        return self.tag_handles[handle] + suffix
+    # def transform_tag(self, handle: Any, suffix: Any) -> Any:
+    #     return self.tag_handles[handle] + suffix
+
+    def select_tag_transform(self, tag: Tag) -> None:
+        if tag is None:
+            return
+        tag.select_transform(False)
 
     def parse_node(self, block: bool = False, indentless_sequence: bool = False) -> Any:
         if self.scanner.check_token(AliasToken):
@@ -370,39 +380,38 @@ class Parser:
                 token = self.scanner.get_token()
                 tag_mark = token.start_mark
                 end_mark = token.end_mark
-                tag = token.value
+                # tag = token.value
+                tag = Tag(
+                    handle=token.value[0], suffix=token.value[1], handles=self.tag_handles,
+                )
         elif self.scanner.check_token(TagToken):
             token = self.scanner.get_token()
+            try:
+                self.move_token_comment(token)
+            except NotImplementedError:
+                pass
             start_mark = tag_mark = token.start_mark
             end_mark = token.end_mark
-            tag = token.value
+            # tag = token.value
+            tag = Tag(handle=token.value[0], suffix=token.value[1], handles=self.tag_handles)
             if self.scanner.check_token(AnchorToken):
                 token = self.scanner.get_token()
                 start_mark = tag_mark = token.start_mark
                 end_mark = token.end_mark
                 anchor = token.value
         if tag is not None:
-            handle, suffix = tag
-            if handle is not None:
-                if handle not in self.tag_handles:
-                    raise ParserError(
-                        'while parsing a node',
-                        start_mark,
-                        f'found undefined tag handle {handle!r}',
-                        tag_mark,
-                    )
-                tag = self.transform_tag(handle, suffix)
-            else:
-                tag = suffix
-        # if tag == '!':
-        #     raise ParserError("while parsing a node", start_mark,
-        #             "found non-specific tag '!'", tag_mark,
-        #      "Please check 'http://pyyaml.org/wiki/YAMLNonSpecificTag'
-        #     and share your opinion.")
+            self.select_tag_transform(tag)
+            if tag.check_handle():
+                raise ParserError(
+                    'while parsing a node',
+                    start_mark,
+                    f'found undefined tag handle {tag.handle!r}',
+                    tag_mark,
+                )
         if start_mark is None:
             start_mark = end_mark = self.scanner.peek_token().start_mark
         event = None
-        implicit = tag is None or tag == '!'
+        implicit = tag is None or str(tag) == '!'
         if indentless_sequence and self.scanner.check_token(BlockEntryToken):
             comment = None
             pt = self.scanner.peek_token()
@@ -410,12 +419,15 @@ class Parser:
                 if pt.comment and pt.comment[0]:
                     comment = [pt.comment[0], []]
                     pt.comment[0] = None
+                elif pt.comment and pt.comment[0] is None and pt.comment[1]:
+                    comment = [None, pt.comment[1]]
+                    pt.comment[1] = None
             elif self.loader:
                 if pt.comment:
                     comment = pt.comment
             end_mark = self.scanner.peek_token().end_mark
             event = SequenceStartEvent(
-                anchor, tag, implicit, start_mark, end_mark, flow_style=False, comment=comment
+                anchor, tag, implicit, start_mark, end_mark, flow_style=False, comment=comment,
             )
             self.state = self.parse_indentless_sequence_entry
             return event
@@ -424,17 +436,16 @@ class Parser:
             token = self.scanner.get_token()
             # self.scanner.peek_token_same_line_comment(token)
             end_mark = token.end_mark
-            if (token.plain and tag is None) or tag == '!':
-                implicit = (True, False)
+            if (token.plain and tag is None) or str(tag) == '!':
+                dimplicit = (True, False)
             elif tag is None:
-                implicit = (False, True)
+                dimplicit = (False, True)
             else:
-                implicit = (False, False)
-            # nprint('se', token.value, token.comment)
+                dimplicit = (False, False)
             event = ScalarEvent(
                 anchor,
                 tag,
-                implicit,
+                dimplicit,
                 token.value,
                 start_mark,
                 end_mark,
@@ -479,14 +490,14 @@ class Parser:
                 comment = pt.split_old_comment()
             # nprint('pt1', comment)
             event = SequenceStartEvent(
-                anchor, tag, implicit, start_mark, end_mark, flow_style=False, comment=comment
+                anchor, tag, implicit, start_mark, end_mark, flow_style=False, comment=comment,
             )
             self.state = self.parse_block_sequence_first_entry
         elif block and self.scanner.check_token(BlockMappingStartToken):
             end_mark = self.scanner.peek_token().start_mark
             comment = self.scanner.peek_token().comment
             event = MappingStartEvent(
-                anchor, tag, implicit, start_mark, end_mark, flow_style=False, comment=comment
+                anchor, tag, implicit, start_mark, end_mark, flow_style=False, comment=comment,
             )
             self.state = self.parse_block_mapping_first_key
         elif anchor is not None or tag is not None:
@@ -554,7 +565,7 @@ class Parser:
             token = self.scanner.get_token()
             self.move_token_comment(token)
             if not self.scanner.check_token(
-                BlockEntryToken, KeyToken, ValueToken, BlockEndToken
+                BlockEntryToken, KeyToken, ValueToken, BlockEndToken,
             ):
                 self.states.append(self.parse_indentless_sequence_entry)
                 return self.parse_block_node()
@@ -672,7 +683,7 @@ class Parser:
             if self.scanner.check_token(KeyToken):
                 token = self.scanner.peek_token()
                 event: Any = MappingStartEvent(
-                    None, None, True, token.start_mark, token.end_mark, flow_style=True
+                    None, None, True, token.start_mark, token.end_mark, flow_style=True,
                 )
                 self.state = self.parse_flow_sequence_entry_mapping_key
                 return event
@@ -740,7 +751,7 @@ class Parser:
             if self.scanner.check_token(KeyToken):
                 token = self.scanner.get_token()
                 if not self.scanner.check_token(
-                    ValueToken, FlowEntryToken, FlowMappingEndToken
+                    ValueToken, FlowEntryToken, FlowMappingEndToken,
                 ):
                     self.states.append(self.parse_flow_mapping_value)
                     return self.parse_flow_node()
@@ -748,7 +759,7 @@ class Parser:
                     self.state = self.parse_flow_mapping_value
                     return self.process_empty_scalar(token.end_mark)
             elif self.resolver.processing_version > (1, 1) and self.scanner.check_token(
-                ValueToken
+                ValueToken,
             ):
                 self.state = self.parse_flow_mapping_value
                 return self.process_empty_scalar(self.scanner.peek_token().end_mark)
@@ -783,7 +794,7 @@ class Parser:
         return ScalarEvent(None, None, (True, False), "", mark, mark, comment=comment)
 
     def move_token_comment(
-        self, token: Any, nt: Optional[Any] = None, empty: Optional[bool] = False
+        self, token: Any, nt: Optional[Any] = None, empty: Optional[bool] = False,
     ) -> Any:
         pass
 
@@ -791,27 +802,13 @@ class Parser:
 class RoundTripParser(Parser):
     """roundtrip is a safe loader, that wants to see the unmangled tag"""
 
-    def transform_tag(self, handle: Any, suffix: Any) -> Any:
-        # return self.tag_handles[handle]+suffix
-        if handle == '!!' and suffix in (
-            'null',
-            'bool',
-            'int',
-            'float',
-            'binary',
-            'timestamp',
-            'omap',
-            'pairs',
-            'set',
-            'str',
-            'seq',
-            'map',
-        ):
-            return Parser.transform_tag(self, handle, suffix)
-        return handle + suffix
+    def select_tag_transform(self, tag: Tag) -> None:
+        if tag is None:
+            return
+        tag.select_transform(True)
 
     def move_token_comment(
-        self, token: Any, nt: Optional[Any] = None, empty: Optional[bool] = False
+        self, token: Any, nt: Optional[Any] = None, empty: Optional[bool] = False,
     ) -> Any:
         token.move_old_comment(self.scanner.peek_token() if nt is None else nt, empty=empty)
 
@@ -823,7 +820,7 @@ class RoundTripParserSC(RoundTripParser):
     # if self.loader.comment_handling is not None
 
     def move_token_comment(
-        self: Any, token: Any, nt: Any = None, empty: Optional[bool] = False
+        self: Any, token: Any, nt: Any = None, empty: Optional[bool] = False,
     ) -> None:
         token.move_new_comment(self.scanner.peek_token() if nt is None else nt, empty=empty)
 
@@ -833,8 +830,8 @@ class RoundTripParserSC(RoundTripParser):
             return None
         if not comment[0]:
             return None
-        if comment[0][0] != line + 1:
-            nprintf('>>>dcxxx', comment, line)
+        # if comment[0][0] != line + 1:
+        #     nprintf('>>>dcxxx', comment, line)
         assert comment[0][0] == line + 1
         # if comment[0] - line > 1:
         #     return
