@@ -5,6 +5,7 @@ import re
 from typing import Any, Dict, List, Union, Text, Optional  # NOQA
 from ruamel.yaml.compat import VersionType  # NOQA
 
+from ruamel.yaml.tag import Tag
 from ruamel.yaml.compat import _DEFAULT_YAML_VERSION  # NOQA
 from ruamel.yaml.error import *  # NOQA
 from ruamel.yaml.nodes import MappingNode, ScalarNode, SequenceNode  # NOQA
@@ -102,9 +103,9 @@ class ResolverError(YAMLError):
 
 class BaseResolver:
 
-    DEFAULT_SCALAR_TAG = 'tag:yaml.org,2002:str'
-    DEFAULT_SEQUENCE_TAG = 'tag:yaml.org,2002:seq'
-    DEFAULT_MAPPING_TAG = 'tag:yaml.org,2002:map'
+    DEFAULT_SCALAR_TAG = Tag(suffix='tag:yaml.org,2002:str')
+    DEFAULT_SEQUENCE_TAG = Tag(suffix='tag:yaml.org,2002:seq')
+    DEFAULT_MAPPING_TAG = Tag(suffix='tag:yaml.org,2002:map')
 
     yaml_implicit_resolvers: Dict[Any, Any] = {}
     yaml_path_resolvers: Dict[Any, Any] = {}
@@ -129,9 +130,9 @@ class BaseResolver:
     def add_implicit_resolver_base(cls, tag: Any, regexp: Any, first: Any) -> None:
         if 'yaml_implicit_resolvers' not in cls.__dict__:
             # deepcopy doesn't work here
-            cls.yaml_implicit_resolvers = dict(
-                (k, cls.yaml_implicit_resolvers[k][:]) for k in cls.yaml_implicit_resolvers
-            )
+            cls.yaml_implicit_resolvers = {
+                k: cls.yaml_implicit_resolvers[k][:] for k in cls.yaml_implicit_resolvers
+            }
         if first is None:
             first = [None]
         for ch in first:
@@ -141,9 +142,9 @@ class BaseResolver:
     def add_implicit_resolver(cls, tag: Any, regexp: Any, first: Any) -> None:
         if 'yaml_implicit_resolvers' not in cls.__dict__:
             # deepcopy doesn't work here
-            cls.yaml_implicit_resolvers = dict(
-                (k, cls.yaml_implicit_resolvers[k][:]) for k in cls.yaml_implicit_resolvers
-            )
+            cls.yaml_implicit_resolvers = {
+                k: cls.yaml_implicit_resolvers[k][:] for k in cls.yaml_implicit_resolvers
+            }
         if first is None:
             first = [None]
         for ch in first:
@@ -236,7 +237,7 @@ class BaseResolver:
         self.resolver_prefix_paths.pop()
 
     def check_resolver_prefix(
-        self, depth: int, path: Any, kind: Any, current_node: Any, current_index: Any
+        self, depth: int, path: Any, kind: Any, current_node: Any, current_index: Any,
     ) -> bool:
         node_check, index_check = path[depth - 1]
         if isinstance(node_check, str):
@@ -268,14 +269,14 @@ class BaseResolver:
             resolvers += self.yaml_implicit_resolvers.get(None, [])
             for tag, regexp in resolvers:
                 if regexp.match(value):
-                    return tag
+                    return Tag(suffix=tag)
             implicit = implicit[1]
         if bool(self.yaml_path_resolvers):
             exact_paths = self.resolver_exact_paths[-1]
             if kind in exact_paths:
-                return exact_paths[kind]
+                return Tag(suffix=exact_paths[kind])
             if None in exact_paths:
-                return exact_paths[None]
+                return Tag(suffix=exact_paths[None])
         if kind is ScalarNode:
             return self.DEFAULT_SCALAR_TAG
         elif kind is SequenceNode:
@@ -306,7 +307,7 @@ class VersionedResolver(BaseResolver):
     """
 
     def __init__(
-        self, version: Optional[VersionType] = None, loader: Any = None, loadumper: Any = None
+        self, version: Optional[VersionType] = None, loader: Any = None, loadumper: Any = None,
     ) -> None:
         if loader is None and loadumper is not None:
             loader = loadumper
@@ -315,7 +316,7 @@ class VersionedResolver(BaseResolver):
         self._version_implicit_resolver: Dict[Any, Any] = {}
 
     def add_version_implicit_resolver(
-        self, version: VersionType, tag: Any, regexp: Any, first: Any
+        self, version: VersionType, tag: Any, regexp: Any, first: Any,
     ) -> None:
         if first is None:
             first = [None]
@@ -329,6 +330,7 @@ class VersionedResolver(BaseResolver):
         if isinstance(version, list):
             return tuple(version)
         # assume string
+        assert isinstance(version, str)
         return tuple(map(int, version.split('.')))
 
     @property
@@ -354,14 +356,14 @@ class VersionedResolver(BaseResolver):
             resolvers += self.versioned_resolver.get(None, [])
             for tag, regexp in resolvers:
                 if regexp.match(value):
-                    return tag
+                    return Tag(suffix=tag)
             implicit = implicit[1]
         if bool(self.yaml_path_resolvers):
             exact_paths = self.resolver_exact_paths[-1]
             if kind in exact_paths:
-                return exact_paths[kind]
+                return Tag(suffix=exact_paths[kind])
             if None in exact_paths:
-                return exact_paths[None]
+                return Tag(suffix=exact_paths[None])
         if kind is ScalarNode:
             return self.DEFAULT_SCALAR_TAG
         elif kind is SequenceNode:
